@@ -4,6 +4,7 @@ __all__ = ['request']
 
 import asyncio
 import base64
+import collections
 import email.message
 import functools
 import http.client
@@ -14,7 +15,9 @@ import inspect
 import itertools
 import mimetypes
 import os
+import random
 import ssl
+import time
 import uuid
 import urllib.parse
 
@@ -34,6 +37,7 @@ def request(method, url, *,
             encoding='utf-8',
             version=(1, 1),
             timeout=None,
+            conn_timeout=None,
             compress=None,
             chunked=None,
             session=None,
@@ -53,6 +57,8 @@ def request(method, url, *,
        for multipart encoding upload
     auth: (optional) Auth tuple to enable Basic HTTP Auth
     timeout: (optional) Float describing the timeout of the request
+    conn_timeout: (optional) Float describing the timeout
+       of the host connection
     allow_redirects: (optional) Boolean. Set to True if POST/PUT/DELETE
        redirect following is allowed.
     compress: Boolean. Set to True if request has to be compressed
@@ -85,18 +91,29 @@ def request(method, url, *,
             verify_ssl=verify_ssl, loop=loop)
 
         if session is None:
-            conn = start(req, loop)
+            conn = _connect(req, loop)
         else:
             conn = session.start(req, loop)
 
-        if timeout:
-            # connection timeout
-            conn = asyncio.wait_for(conn, timeout, loop=loop)
+        if conn_timeout is None and timeout is not None:
+            conn_timeout = timeout
 
+        conn_task = asyncio.async(conn, loop=loop)
         try:
-            resp = yield from conn
+            if conn_timeout:
+                transport, proto, wrp = yield from asyncio.wait_for(
+                    conn_task, conn_timeout, loop=loop)
+            else:
+                transport, proto, wrp = yield from conn_task
+
+            resp = yield from _make_request(
+                transport, proto, req, wrp, timeout, loop)
         except asyncio.TimeoutError:
-            raise asyncio.TimeoutError from None
+            raise aiohttp.TimeoutError from None
+        except OSError as exc:
+            raise aiohttp.ConnectionError(exc)
+        finally:
+            conn_task.cancel()
 
         # redirects
         if resp.status in (301, 302) and allow_redirects:
@@ -124,19 +141,166 @@ def request(method, url, *,
 
 
 @asyncio.coroutine
-def start(req, loop):
-    transport, p = yield from loop.create_connection(
+def _connect(req, loop):
+    transport, proto = yield from loop.create_connection(
         functools.partial(aiohttp.StreamProtocol, loop=loop),
         req.host, req.port, ssl=req.ssl)
+    wrp = TransportWrapper(transport)
+    return transport, proto, wrp
 
+
+@asyncio.coroutine
+def _make_request(transport, proto, req, wrapper, timeout, loop):
     try:
         resp = req.send(transport)
-        yield from resp.start(p, transport)
+        if timeout:
+            yield from asyncio.wait_for(
+                resp.start(proto, wrapper), timeout, loop=loop)
+        else:
+            yield from resp.start(proto, wrapper)
     except:
         transport.close()
         raise
+    else:
+        return resp
 
-    return resp
+
+class TransportWrapper:
+
+    def __init__(self, transport):
+        self.transport = transport
+
+    def close(self, force=False):
+        self.transport.close()
+
+
+class HttpClient:
+    """Allow to use mutiple hosts with same path. And automatically
+    mark failed hosts.
+    """
+
+    def __init__(self, hosts, *,
+                 method=None, path=None, ssl=False,
+                 timeout=None, conn_timeout=None, failed_timeout=5.0,
+                 loop=None):
+        super().__init__()
+
+        if isinstance(hosts, str):
+            hosts = (hosts,)
+
+        if not hosts:
+            raise ValueError('Hosts are required')
+
+        self._hosts = []
+        for host in hosts:
+            if isinstance(host, str):
+                if ':' in host:
+                    host, port = host.split(':')
+                    try:
+                        port = int(port)
+                    except:
+                        raise ValueError('Port has to be integer: %s' % host)
+                else:
+                    port = 80
+            else:
+                host, port = host
+
+            self._hosts.append((host, port))
+
+        self._method = method
+        self._path = path
+        self._ssl = ssl
+        self._timeout = timeout
+        self._conn_timeout = conn_timeout
+        self._schema = 'https' if ssl else 'http'
+        self._session = aiohttp.Session()
+
+        self._failed = collections.deque()
+        self._failed_handle = None
+        self._failed_timeout = failed_timeout
+
+        if loop is None:
+            loop = asyncio.get_event_loop()
+        self._loop = loop
+
+    def _resurrect_failed(self):
+        now = int(time.time())
+
+        while self._failed:
+            host, port, ts = self._failed[0]
+            if (now - ts) >= self._failed_timeout:
+                self._hosts.append((host, port))
+                self._failed.popleft()
+            else:
+                break
+
+        if self._failed:
+            self._failed_handle = self._loop.call_later(
+                self._failed_timeout, self._resurrect_failed)
+        else:
+            self._failed_handle = None
+
+    @asyncio.coroutine
+    def request(self, method=None, path=None, *,
+                params=None,
+                data=None,
+                headers=None,
+                cookies=None,
+                files=None,
+                auth=None,
+                allow_redirects=True,
+                max_redirects=10,
+                encoding='utf-8',
+                version=(1, 1),
+                compress=None,
+                timeout=None,
+                conn_timeout=None,
+                chunked=None,
+                verify_ssl=True):
+
+        if method is None:
+            method = self._method
+        if path is None:
+            path = self._path
+        if timeout is None:
+            timeout = self._timeout
+        if conn_timeout is None:
+            conn_timeout = self._conn_timeout
+
+        # if all hosts marked as failed try first from failed
+        if not self._hosts:
+            self._hosts.append(self._failed.popleft()[:-1])
+
+        hosts = self._hosts
+
+        while hosts:
+            idx = random.randint(0, len(hosts)-1)
+
+            host, port = hosts[idx]
+            url = urllib.parse.urljoin(
+                '%s://%s:%s' % (self._schema, host, port), path)
+
+            try:
+                resp = yield from request(
+                    method, url,
+                    params=params, data=data, headers=headers,
+                    cookies=cookies, files=files, auth=auth,
+                    allow_redirects=allow_redirects,
+                    max_redirects=max_redirects,
+                    encoding=encoding, version=version,
+                    timeout=timeout, conn_timeout=conn_timeout,
+                    compress=compress, chunked=chunked, verify_ssl=verify_ssl,
+                    session=self._session, loop=self._loop)
+            except (aiohttp.ConnectionError, aiohttp.TimeoutError):
+                hosts.pop(idx)
+                self._failed.append((host, port, int(time.time())))
+                if not self._failed_handle:
+                    self._failed_handle = self._loop.call_later(
+                        self._failed_timeout, self._resurrect_failed)
+            else:
+                return resp
+
+        raise aiohttp.ConnectionError('All hosts are unreachable.')
 
 
 class HttpRequest:
@@ -152,6 +316,8 @@ class HttpRequest:
 
     body = b''
     auth = None
+    response = None
+
     _writer = None  # async task for streaming body
 
     def __init__(self, method, url, *,
@@ -475,7 +641,8 @@ class HttpRequest:
 
             request.write_eof()
 
-        return HttpResponse(self.method, self.path, self.host)
+        self.response = HttpResponse(self.method, self.path, self.host)
+        return self.response
 
     @asyncio.coroutine
     def close(self):
@@ -554,7 +721,7 @@ class HttpResponse(http.client.HTTPMessage):
 
     def close(self, force=False):
         if self.transport is not None:
-            self.transport.close()
+            self.transport.close(force)
             self.transport = None
 
     @asyncio.coroutine
