@@ -40,6 +40,39 @@ class HttpClientFunctionalTests(unittest.TestCase):
                 self.assertEqual(content1, content2)
                 r.close()
 
+    def test_client_HTTP_200_OK_METHOD(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            for meth in ('get', 'post', 'put', 'delete', 'head'):
+                host = httpd.url().split('://', 1)[-1]
+                httpclient = client.HttpClient(host, loop=self.loop)
+
+                r = self.loop.run_until_complete(
+                    httpclient.request(meth, '/method/%s' % meth))
+                content1 = self.loop.run_until_complete(r.read())
+                content2 = self.loop.run_until_complete(r.read())
+                content = content1.decode()
+
+                self.assertEqual(r.status, 200)
+                self.assertIn('"method": "%s"' % meth.upper(), content)
+                self.assertEqual(content1, content2)
+                r.close()
+
+    def test_client_defaults_HTTP_200_OK_METHOD(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            host = httpd.url().split('://', 1)[-1]
+            httpclient = client.HttpClient(
+                host, method='GET', path='/method/get', loop=self.loop)
+
+            r = self.loop.run_until_complete(httpclient.request())
+            content1 = self.loop.run_until_complete(r.read())
+            content2 = self.loop.run_until_complete(r.read())
+            content = content1.decode()
+
+            self.assertEqual(r.status, 200)
+            self.assertIn('"method": "GET"', content)
+            self.assertEqual(content1, content2)
+            r.close()
+
     def test_HTTP_200_OK_METHOD_ssl(self):
         with test_utils.run_server(self.loop, use_ssl=True) as httpd:
             for meth in ('get', 'post', 'put', 'delete', 'head'):
@@ -443,7 +476,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
 
     def test_request_conn_error(self):
         self.assertRaises(
-            OSError,
+            aiohttp.ConnectionError,
             self.loop.run_until_complete,
             client.request('get', 'http://0.0.0.0:1',
                            timeout=0.1, loop=self.loop))
